#! /bin/sh
#######################################
### Extensions and Plugin Installer ###
#######################################

echo_with_flag()
{
    # if [ "$VERBOSE" == "1" ]
    # then
        echo "$1"
    # fi
}

opera_extensionExists()
{
    opera_exists
    if [ $? == 1 ]; then
        return 1
    fi
        
    OLDIFS="$IFS"
    IFS=$'\n'
    test -e "$OPERA_PREF_PATH"
    if [ $? == 0 ]; then
        echo_with_flag "OPERA Extension exists"
        return 0
    fi
    IFS=$OLDIFS
    
    echo_with_flag "OPERA Extension does not exist"                    
    return 1
}

opera_exists()
{
    OPERA_BROWSERS=$(mdfind "kMDItemCFBundleIdentifier == 'com.operasoftware.Opera'")
    if [ "$OPERA_BROWSERS" == "" ]; then
        echo_with_flag "OPERA does not exist"
        return 1
    fi
    
    echo_with_flag "OPERA exists"
    return 0
}


opera_rightVersion()
{
	ORIGINAL_IFS=$IFS
	IFS=$'\n'
	if [ -e /usr/libexec/PlistBuddy ]
	then
		for BROWSER in $(mdfind "kMDItemCFBundleIdentifier == 'com.operasoftware.Opera'")
		do
		 	echo_with_flag "$BROWSER"
		 	if [ -e "$BROWSER"/Contents/Info.plist ]
		 	then
		 		VERSION=$(/usr/libexec/PlistBuddy -c "Print :CFBundleShortVersionString" "$BROWSER"/Contents/Info.plist | cut -d . -f1)
		 		echo_with_flag "version is $VERSION"
		 		if [ $VERSION -ge 6 ]
		 		then
		 			echo_with_flag "good version"
		 			return 0
		 		else
		 			echo_with_flag "bad version"
		 		fi
		 	else
		 		echo_with_flag "no plist"
		 	fi
		done;
	fi
	IFS=$ORIGINAL_IFS
	return 1
}

old_opera_uninstall()
{
	if [ "$OLD_OPERA_PREF_PATH" == "" ]; then
		return 0
	fi
	
    echo_with_flag "Uninstalling Kaspersky OPERA URL Advisor Extension"
    
    OLDIFS=$IFS
    IFS=$'\n'
    rm "$OLD_OPERA_PREF_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove $OLD_OPERA_PREF_PATH from External Extensions folder"
        return 1
    fi
    IFS=$OLDIFS
    
    echo_with_flag "$OLD_OPERA_PREF_PATH removed from External Extensions folder"
    
    return 0
}

opera_uninstall()
{
	echo_with_flag "Uninstalling old Kaspersky OPERA URL Advisor Extension"
	old_opera_uninstall
	echo_with_flag "Uninstalling old Kaspersky OPERA URL Advisor Extension done"
	
    opera_extensionExists
    if [ $? == 1 ]; then
        return 1
    fi

    echo_with_flag "Uninstalling Kaspersky OPERA URL Advisor Extension"
    
    OLDIFS=$IFS
    IFS=$'\n'
    rm "$OPERA_PREF_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove $OPERA_PREF_PATH from External Extensions folder"
        return 1
    fi
    IFS=$OLDIFS
    
    echo_with_flag "$OPERA_PREF_PATH removed from External Extensions folder"
    
    check_and_install_or_plugin_uninstall
    
    return 0
}

opera_install()
{
    echo_with_flag "Installing Kaspersky OPERA URL Advisor Extension"
	opera_exists
    if [ $? == 1 ]; then
        return 0
    fi
    
    opera_uninstall
    
    CHROMETEXT="{\"external_crx\": \"$OPERA_SOURCE_PATH\", \"external_version\": \"$OPERA_EXT_VERSION\"}"
    
    OLDIFS=$IFS
    IFS=$'\n'
    # make directory for all users $CHROME_MAGIC
    mkdir -v -p "$OPERA_DEST_PATH"
    echo_with_flag "chown $CURR_USER $CHROME_DEST_PATH"    
    chown $CURR_USER "$OPERA_DEST_ROOT_PATH"
    chown $CURR_USER "$OPERA_DEST_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot create External Extensions folder"
        return 1
    fi
    SWAPFILE="$(mktemp /tmp/XXXXXX)"
    echo "$CHROMETEXT" >> "$SWAPFILE";
    mv "$SWAPFILE" "$OPERA_PREF_PATH"    
    echo_with_flag "chown $CURR_USER $OPERA_PREF_PATH"        
    chown $CURR_USER "$OPERA_PREF_PATH"
    IFS=$OLDIFS
    
    plugin_install
    
    return 0
}

yandex_extensionExists()
{
    yandex_exists
    if [ $? == 1 ]; then
        return 1
    fi
        
    OLDIFS="$IFS"
    IFS=$'\n'
    test -e "$YANDEX_PREF_PATH"
    if [ $? == 0 ]; then
        echo_with_flag "Yandex Browser Extension exists"
        return 0
    fi
    IFS=$OLDIFS
    
    echo_with_flag "Yandex Browser Extension does not exist"                    
    return 1
}

yandex_exists()
{
    YANDEX_BROWSERS=$(mdfind "kMDItemCFBundleIdentifier == 'ru.yandex.desktop.yandex-browser'")
    if [ "$YANDEX_BROWSERS" == "" ]; then
        echo_with_flag "Yandex Browser does not exist"
        return 1
    fi
    
    echo_with_flag "Yandex Browser exists"
    return 0
}


yandex_rightVersion()
{
	ORIGINAL_IFS=$IFS
	IFS=$'\n'
	if [ -e /usr/libexec/PlistBuddy ]
	then
		for BROWSER in $(mdfind "kMDItemCFBundleIdentifier == 'ru.yandex.desktop.yandex-browser'")
		do
		 	echo_with_flag "$BROWSER"
		 	if [ -e "$BROWSER"/Contents/Info.plist ]
		 	then
		 		VERSION=$(/usr/libexec/PlistBuddy -c "Print :CFBundleShortVersionString" "$BROWSER"/Contents/Info.plist | cut -d . -f1)
		 		echo_with_flag "version is $VERSION"
		 		if [ $VERSION -ge 6 ]
		 		then
		 			echo_with_flag "good version"
		 			return 0
		 		else
		 			echo_with_flag "bad version"
		 		fi
		 	else
		 		echo_with_flag "no plist"
		 	fi
		done;
	fi
	IFS=$ORIGINAL_IFS
	return 1
}

old_yandex_uninstall()
{
	if [ "$OLD_YANDEX_PREF_PATH" == "" ]; then
		return 0
	fi
	
    echo_with_flag "Uninstalling Kaspersky Yandex URL Advisor Extension"
    
    OLDIFS=$IFS
    IFS=$'\n'
    rm "$OLD_YANDEX_PREF_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove $OLD_YANDEX_PREF_PATH from External Extensions folder"
        return 1
    fi
    IFS=$OLDIFS
    
    echo_with_flag "$OLD_YANDEX_PREF_PATH removed from External Extensions folder"
    
    return 0
}

yandex_uninstall()
{
	echo_with_flag "Uninstalling old Kaspersky Yandex URL Advisor Extension"
	old_yandex_uninstall
	echo_with_flag "Uninstalling old Kaspersky Yandex URL Advisor Extension done"
	
    yandex_extensionExists
    if [ $? == 1 ]; then
        return 1
    fi

    echo_with_flag "Uninstalling Kaspersky Yandex URL Advisor Extension"
    
    OLDIFS=$IFS
    IFS=$'\n'
    rm "$YANDEX_PREF_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove $YANDEX_PREF_PATH from External Extensions folder"
        return 1
    fi
    IFS=$OLDIFS
    
    echo_with_flag "$YANDEX_PREF_PATH removed from External Extensions folder"
    
    check_and_install_or_plugin_uninstall
    
    return 0
}

yandex_install()
{
    echo_with_flag "Installing Kaspersky Yandex URL Advisor Extension"
	yandex_exists
    if [ $? == 1 ]; then
        return 0
    fi
    
    yandex_uninstall
    
    CHROMETEXT="{\"external_crx\": \"$YANDEX_SOURCE_PATH\", \"external_version\": \"$YANDEX_EXT_VERSION\"}"
    
    OLDIFS=$IFS
    IFS=$'\n'
    # make directory for all users $CHROME_MAGIC
    mkdir -v -p "$YANDEX_DEST_PATH"
    echo_with_flag "chown $CURR_USER $YANDEX_DEST_PATH"    
    chown $CURR_USER "$YANDEX_DEST_ROOT_PATH"
    chown $CURR_USER "$YANDEX_DEST_ROOT_PATH/YandexBrowser"
    chown $CURR_USER "$YANDEX_DEST_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot create External Extensions folder"
        return 1
    fi
    SWAPFILE="$(mktemp /tmp/XXXXXX)"
    echo "$CHROMETEXT" >> "$SWAPFILE";
    mv "$SWAPFILE" "$YANDEX_PREF_PATH"    
    echo_with_flag "chown $CURR_USER $YANDEX_PREF_PATH"        
    chown $CURR_USER "$YANDEX_PREF_PATH"
    IFS=$OLDIFS
    
    plugin_install
    
    return 0
}

firefox_extensionExists()
{
    firefox_exists
    if [ $? == 1 ]; then
        return 1
    fi

    if [ -e "$FIREFOX_EXT_PATH" ]; then
        echo_with_flag "Firefox Extension exists"
        return 0
    fi

    echo_with_flag "Firefox Extension doesn't exist"    
    return 1
}

firefox_exists()
{
    FIREFOX_BROWSERS=$(mdfind "kMDItemCFBundleIdentifier == 'org.mozilla.firefox'")
    if [ "$FIREFOX_BROWSERS" = "" ]; then
        echo_with_flag "Mozilla Firefox does not exist"
        return 1
    fi
    
    echo_with_flag "Mozilla Firefox exists"
    return 0
}

firefox_rightVersion()
{
	ORIGINAL_IFS=$IFS
	IFS=$'\n'
	if [ -e /usr/libexec/PlistBuddy ]
	then
		for BROWSER in $(mdfind "kMDItemCFBundleIdentifier == 'org.mozilla.firefox'")
		do
		 	echo_with_flag "$BROWSER"
		 	if [ -e "$BROWSER"/Contents/Info.plist ]
		 	then
		 		VERSION=$(/usr/libexec/PlistBuddy -c "Print :CFBundleShortVersionString" "$BROWSER"/Contents/Info.plist | cut -d . -f1)
		 		echo_with_flag "version is $VERSION"
		 		if [ $VERSION -ge 4 ]
		 		then
		 			echo_with_flag "good version"
		 			return 0
		 		else
		 			echo_with_flag "bad version"
		 		fi
		 	else
		 		echo_with_flag "no plist"
		 	fi
		done;
	fi
	IFS=$ORIGINAL_IFS
	return 1
}

old_firefox_uninstall()
{
	if [ "$OLD_FIREFOX_EXT_PATH" == "" ]; then
		return 0
	fi
	
    echo_with_flag "Uninstalling Kaspersky Firefox URL Advisor Extension"
    
    rm "$OLD_FIREFOX_EXT_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove extension from Library"
        return 1
    fi     
}


firefox_uninstall()
{
    echo_with_flag "Uninstalling Old Kaspersky Firefox URL Advisor Extension"
	old_firefox_uninstall
    echo_with_flag "Uninstalled Old Kaspersky Firefox URL Advisor Extension"
	
    firefox_extensionExists
    if [ $? == 1 ]; then
        return 1
    fi
    
    echo_with_flag "Uninstalling Kaspersky Firefox URL Advisor Extension"
    
    rm "$FIREFOX_EXT_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove extension from Library"
        return 1
    fi     
    
    check_and_install_or_plugin_uninstall
}

firefox_install()
{
	firefox_exists
    if [ $? == 1 ]; then
        return 0
    fi
        
    firefox_uninstall
    
    echo_with_flag "Installing Kaspersky Firefox URL Advisor Extension"
 
    SWAPFILE="$(mktemp /tmp/XXXXXX)"
    # echo_with_flag "$FIREFOX_SOURCE_PATH" >> "$SWAPFILE";
    cat "$FIREFOX_SOURCE_PATH" >> "$SWAPFILE"
    mkdir -pv "$FIREFOX_DEST_PATH"
    echo_with_flag "chown $CURR_USER $FIREFOX_DEST_PATH"
    chown $CURR_USER "$FIREFOX_DEST_ROOT_PATH"    
    chown $CURR_USER "$FIREFOX_DEST_PATH"
    mv "$SWAPFILE" "$FIREFOX_EXT_PATH"    
    if [ $? != 0 ]; then
        echo_with_flag "Cannot move extension to Library"
        return 1
    fi
    echo_with_flag "chown $CURR_USER $FIREFOX_EXT_PATH"
    chown $CURR_USER "$FIREFOX_EXT_PATH"                
    chmod a+r "$FIREFOX_EXT_PATH"    
    
    plugin_install
}

chrome_extensionExists()
{
    chrome_exists
    if [ $? == 1 ]; then
        return 1
    fi
        
    OLDIFS="$IFS"
    IFS=$'\n'
    test -e "$CHROME_PREF_PATH"
    if [ $? == 0 ]; then
        echo_with_flag "Google Chrome Extension exists"
        return 0
    fi
    IFS=$OLDIFS
    
    echo_with_flag "Google Chrome Extension does not exist"                    
    return 1
}

chrome_exists()
{
    CHROME_BROWSERS=$(mdfind "kMDItemCFBundleIdentifier == 'com.google.Chrome'")
    if [ "$CHROME_BROWSERS" == "" ]; then
        echo_with_flag "Google Chrome does not exist"
        return 1
    fi
    
    echo_with_flag "Google Chrome exists"
    return 0
}


chrome_rightVersion()
{
	ORIGINAL_IFS=$IFS
	IFS=$'\n'
	if [ -e /usr/libexec/PlistBuddy ]
	then
		for BROWSER in $(mdfind "kMDItemCFBundleIdentifier == 'com.google.Chrome'")
		do
		 	echo_with_flag "$BROWSER"
		 	if [ -e "$BROWSER"/Contents/Info.plist ]
		 	then
		 		VERSION=$(/usr/libexec/PlistBuddy -c "Print :CFBundleShortVersionString" "$BROWSER"/Contents/Info.plist | cut -d . -f1)
		 		echo_with_flag "version is $VERSION"
		 		if [ $VERSION -ge 6 ]
		 		then
		 			echo_with_flag "good version"
		 			return 0
		 		else
		 			echo_with_flag "bad version"
		 		fi
		 	else
		 		echo_with_flag "no plist"
		 	fi
		done;
	fi
	IFS=$ORIGINAL_IFS
	return 1
}

old_chrome_uninstall()
{
	if [ "$OLD_CHROME_PREF_PATH" == "" ]; then
		return 0
	fi	
	
    echo_with_flag "Uninstalling Kaspersky Chrome URL Advisor Extension"
    
    OLDIFS=$IFS
    IFS=$'\n'
    rm "$OLD_CHROME_PREF_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove $OLD_CHROME_PREF_PATH from External Extensions folder"
        return 1
    fi
    IFS=$OLDIFS
    
    echo_with_flag "$OLD_CHROME_PREF_PATH removed from External Extensions folder"
    
    return 0
}

chrome_uninstall()
{
    echo_with_flag "Uninstalling Old Kaspersky Chrome URL Advisor Extension"
	old_chrome_uninstall
    echo_with_flag "Uninstalled Old Kaspersky Chrome URL Advisor Extension"
	
    chrome_extensionExists
    if [ $? == 1 ]; then
        return 1
    fi

    echo_with_flag "Uninstalling Kaspersky Chrome URL Advisor Extension"
    
    OLDIFS=$IFS
    IFS=$'\n'
    rm "$CHROME_PREF_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove $CHROME_PREF_PATH from External Extensions folder"
        return 1
    fi
    IFS=$OLDIFS
    
    echo_with_flag "$CHROME_PREF_PATH removed from External Extensions folder"
    
    check_and_install_or_plugin_uninstall
    
    return 0
}

chrome_install()
{
    echo_with_flag "Installing Kaspersky Chrome URL Advisor Extension"
	chrome_exists
    if [ $? == 1 ]; then
        return 0
    fi
    
    chrome_uninstall
    
    CHROMETEXT="{\"external_crx\": \"$CHROME_SOURCE_PATH\", \"external_version\": \"$CHROME_EXT_VERSION\"}"
    
    OLDIFS=$IFS
    IFS=$'\n'
    # make directory for all users $CHROME_MAGIC
    mkdir -v -p "$CHROME_DEST_PATH"
    echo_with_flag "chown $CURR_USER $CHROME_DEST_PATH"    
    chown $CURR_USER "$CHROME_DEST_ROOT_PATH"
    chown $CURR_USER "$CHROME_DEST_ROOT_PATH/Chrome"
    chown $CURR_USER "$CHROME_DEST_PATH"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot create External Extensions folder"
        return 1
    fi
    SWAPFILE="$(mktemp /tmp/XXXXXX)"
    echo "$CHROMETEXT" >> "$SWAPFILE";
    mv "$SWAPFILE" "$CHROME_PREF_PATH"    
    echo_with_flag "chown $CURR_USER $CHROME_PREF_PATH"        
    chown $CURR_USER "$CHROME_PREF_PATH"
    IFS=$OLDIFS
    
    plugin_install
    
    return 0
}

safari_extensionExists()
{
    safari_exists
    if [ $? == 1 ]; then
        return 1
    fi
	
	if [ "$SAFARI_VERSION_MAJOR" -ge "7" ]; then
		"$SAFARI_KEYCHAIN_TOOL_PATH" checkState "$SAFARI_EXT_NAME" "$SAFARI_EXT_BNDL" "$SAFARI_EXT_BUNDLE_ID"
		if [ $? != 0 ]; then
			echo_with_flag "Apple Safari Extension does not exist in Safari extended preferences"
			return 1
		else
			echo_with_flag "Apple Safari Extension exists in Safari extended preferences"
			return 0
		fi
	elif [ "$SAFARI_VERSION_MAJOR" -ge "6" ]; then
		if [ "$SAFARI_VERSION_MINOR" -ge "2" ]; then
			"$SAFARI_KEYCHAIN_TOOL_PATH" checkState "$SAFARI_EXT_NAME" "$SAFARI_EXT_BNDL" "$SAFARI_EXT_BUNDLE_ID"
			if [ $? != 0 ]; then
				echo_with_flag "Apple Safari Extension does not exist in Safari extended preferences"
				return 1
			else
				echo_with_flag "Apple Safari Extension exists in Safari extended preferences"
				return 0
			fi
		fi
	fi
    
    echo_with_flag "edit: $EXTZPLIST_PATH"
    SWAPFILE="$(mktemp /tmp/XXXXXX)"
    cat "$EXTZPLIST_PATH" > "$SWAPFILE"
    plutil -convert xml1 "$SWAPFILE"
    INSERTPOINT=$(grep -n "$SAFARI_EXT_NAME" "$SWAPFILE" | cut -f1 -d:)
    if [ "$INSERTPOINT" != "" ]; 
    then
        echo_with_flag "Apple Safari Extension exists"
        return 0
    else
        echo_with_flag "Apple Safari Extension does not exist"                
        return 1            
    fi    

    echo_with_flag "Apple Safari Extension does not exist"                    
    return 1
}

safari_exists()
{
    SAFARI_BROWSERS=$(mdfind "kMDItemCFBundleIdentifier == 'com.apple.Safari'")
    if [ "$SAFARI_BROWSERS" == "" ]; then
        echo_with_flag "Apple Safari does not exist"
        return 1
    fi
    
    echo_with_flag "Apple Safari exists"
    return 0
}

safari_rightVersion()
{
	ORIGINAL_IFS=$IFS
	IFS=$'\n'
	if [ -e /usr/libexec/PlistBuddy ]
	then
		for BROWSER in $(mdfind "kMDItemCFBundleIdentifier == 'com.apple.Safari'")
		do
		 	echo_with_flag "$BROWSER"
		 	if [ -e "$BROWSER"/Contents/Info.plist ]
		 	then
		 		VERSION=$(/usr/libexec/PlistBuddy -c "Print :CFBundleShortVersionString" "$BROWSER"/Contents/Info.plist | cut -d . -f1)
		 		echo_with_flag "version is $VERSION"
		 		if [ $VERSION -ge 5 ]
		 		then
		 			echo_with_flag "good version"
		 			return 0
		 		else
		 			echo_with_flag "bad version"
		 		fi
		 	else
		 		echo_with_flag "no plist"
		 	fi
		done;
	fi
	IFS=$ORIGINAL_IFS
	return 1
}

old_safari_uninstall()
{
	if [ "$OLD_SAFARI_EXT_NAME" == "" ]; then
		return 0
	fi
	
    echo_with_flag "Unstalling Kaspersky $OLD_SAFARI_EXT_NAME Extension"
    
    # Delete .safariextz
    rm "$OLD_SAFARI_DEST_PATH"

    # Edit Extensions.plist
    PYTHON_COMMAND_UNINSTALL="uninstallExtension('$OLD_SAFARI_EXT_NAME')"
    python -c "$PYTHON_ENVIRONMENT$PYTHON_COMMAND_UNINSTALL"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove extension $OLD_SAFARI_EXT_NAME"
        return 1
    fi
    
	# find account name by user home dir ($USER and $SUDOUSER are not available)
	if [ "$_HOME" == "" ]; then
		CURR_USER=`python -c "import os,pwd; print [x.pw_name for x in pwd.getpwall() if x.pw_dir == os.getenv('HOME')][0]"`
	else
		CURR_USER=$(basename $_HOME)
	fi
	ACTING_USER=`python -c "import os,pwd; print [x.pw_name for x in pwd.getpwall() if x.pw_dir == os.getenv('HOME')][0]"`
	if [[ "$OSVERMINOR" -ge "10" ]] && [[ "$CURR_USER" == "$ACTING_USER" ]]; then
		"$SAFARI_KEYCHAIN_TOOL_PATH" uninstall "$OLD_SAFARI_EXT_NAME" "$OLD_SAFARI_EXT_BNDL" "$OLD_SAFARI_EXT_BUNDLE_ID"
    fi
	
    return 0
}

safari_uninstall()
{
    echo_with_flag "Unstalling old Kaspersky $SAFARI_EXT_NAME Extension"
	old_safari_uninstall
    echo_with_flag "Unstalling old Kaspersky $SAFARI_EXT_NAME Extension done"
	
    echo_with_flag "Unstalling Kaspersky $SAFARI_EXT_NAME Extension"
    
    # Delete .safariextz
    rm "$SAFARI_DEST_PATH"

    # Edit Extensions.plist
    PYTHON_COMMAND_UNINSTALL="uninstallExtension('$SAFARI_EXT_NAME')"
    python -c "$PYTHON_ENVIRONMENT$PYTHON_COMMAND_UNINSTALL"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot remove extension $SAFARI_EXT_NAME"
        return 1
    fi
    
    check_and_install_or_plugin_uninstall
    
	# find account name by user home dir ($USER and $SUDOUSER are not available)
	if [ "$_HOME" == "" ]; then
		CURR_USER=`python -c "import os,pwd; print [x.pw_name for x in pwd.getpwall() if x.pw_dir == os.getenv('HOME')][0]"`
	else
		CURR_USER=$(basename $_HOME)
	fi
	ACTING_USER=`python -c "import os,pwd; print [x.pw_name for x in pwd.getpwall() if x.pw_dir == os.getenv('HOME')][0]"`
	if [[ "$OSVERMINOR" -ge "10" ]] && [[ "$CURR_USER" == "$ACTING_USER" ]]; then
		"$SAFARI_KEYCHAIN_TOOL_PATH" uninstall "$SAFARI_EXT_NAME" "$SAFARI_EXT_BNDL" "$SAFARI_EXT_BUNDLE_ID"
    fi
	
    return 0
}

safari_install()
{
	safari_exists
    if [ $? == 1 ]; then
        return 0
    fi
    
    # Create symlink .safariextz in ~/Library/Safari/Extensions
    echo_with_flag "from: $SAFARI_SOURCE_PATH"
    echo_with_flag "to: $SAFARI_EXT_PATH"
    echo_with_flag "as: $SAFARI_DEST_PATH"

    mkdir -v -p "$SAFARI_EXT_PATH"
    echo_with_flag "chown $CURR_USER $SAFARI_EXT_PATH"    
    chown $CURR_USER "$SAFARI_DEST_ROOT_PATH"
    chown $CURR_USER "$SAFARI_EXT_PATH"    

    echo_with_flag ln "$SAFARI_SOURCE_PATH" "$SAFARI_DEST_PATH"
    ln -sf "$SAFARI_SOURCE_PATH" "$SAFARI_DEST_PATH"

    if [ $? != 0 ]; 

    then
        echo_with_flag "Failed to copy $SAFARI_EXT_NAME extension"
        return 1;
    fi
    
    PYTHON_COMMAND_INSTALL="installExtension('$SAFARI_EXT_NAME', '$SAFARI_EXT_BNDL')"
    python -c "$PYTHON_ENVIRONMENT$PYTHON_COMMAND_INSTALL"
    if [ $? != 0 ]; then
        echo_with_flag "Cannot install extension $SAFARI_EXT_NAME"
        return 1
    fi
	CURR_GRP=$(id -g -n $CURR_USER)
	[ $? == 0 ] && chown "$CURR_USER":"$CURR_GRP" "$SAFARI_EXT_PATH/Extensions.plist"
        
    echo_with_flag "$SAFARI_EXT_NAME Extensions installed succefully. Restart Safari."

    plugin_install
	
	if [ "$SAFARI_VERSION_MAJOR" -ge "7" ]; then
		"$SAFARI_KEYCHAIN_TOOL_PATH" install "$SAFARI_EXT_NAME" "$SAFARI_EXT_BNDL" "$SAFARI_EXT_BUNDLE_ID"
	elif [ "$SAFARI_VERSION_MAJOR" -ge "6" ]; then
		if [ "$SAFARI_VERSION_MINOR" -ge "2" ]; then
			"$SAFARI_KEYCHAIN_TOOL_PATH" install "$SAFARI_EXT_NAME" "$SAFARI_EXT_BNDL" "$SAFARI_EXT_BUNDLE_ID"
		fi
	fi

    return 0
}

safari_checkState()
{
	safari_exists
    if [ $? == 1 ]; then
        return 2
    fi
    
    safari_rightVersion
     if [ $? == 1 ]; then
        return 3
    fi
        
    python "$SCRIPT_DIR"/shared_extensions_installer.py "$SCRIPT_BROWSER" "extensionExists" "$EXT_VERSION" "$EXT_SRC_PATH" "$EXT_ID" "$APP_ID" "$SELF_NAME" "$_HOME"
    if [ $? == 1 ]; then
        return 4
    fi
	
    safari_extensionExists
    if [ $? == 1 ]; then
        return 4
    fi
	
    return 0;
}

firefox_checkState()
{
	firefox_exists
    if [ $? == 1 ]; then
        return 2
    fi
    
    firefox_rightVersion
    if [ $? == 1 ]; then
        return 3
    fi
    
    python "$SCRIPT_DIR"/shared_extensions_installer.py "$SCRIPT_BROWSER" "extensionExists" "$EXT_VERSION" "$EXT_SRC_PATH" "$EXT_ID" "$APP_ID" "$SELF_NAME" "$_HOME"
    if [ $? == 1 ]; then
        return 4
    fi
	
    firefox_extensionExists
    if [ $? == 1 ]; then
        return 4
    fi
	
    return 0;
}

chrome_checkState()
{
	chrome_exists
    if [ $? == 1 ]; then
        return 2
    fi
    
    chrome_rightVersion
     if [ $? == 1 ]; then
        return 3
    fi
    
    python "$SCRIPT_DIR"/shared_extensions_installer.py "$SCRIPT_BROWSER" "extensionExists" "$EXT_VERSION" "$EXT_SRC_PATH" "$EXT_ID" "$APP_ID" "$SELF_NAME" "$_HOME"
    if [ $? == 1 ]; then
        return 4
    fi
	
    chrome_extensionExists
    if [ $? == 1 ]; then
        return 4
    fi
	
    return 0;
}

yandex_checkState()
{
	yandex_exists
    if [ $? == 1 ]; then
        return 2
    fi
    
    yandex_rightVersion
     if [ $? == 1 ]; then
        return 3
    fi
    
    yandex_extensionExists
    if [ $? == 1 ]; then
        return 4
    fi
    return 0;
}

opera_checkState()
{
	opera_exists
    if [ $? == 1 ]; then
        return 2
    fi
    
    opera_rightVersion
     if [ $? == 1 ]; then
        return 3
    fi
    
    opera_extensionExists
    if [ $? == 1 ]; then
        return 4
    fi
    return 0;
}

plugin_install()
{
    if [ "$PLUGIN_SAFARI_LOAD_POLICY_NAME" == "" ] ; then
        echo "no plugin"
    else
        mkdir -v -p "$PLUGIN_DEST_PATH"
        echo_with_flag "chown $CURR_USER $PLUGIN_DEST_PATH"
        chown -R $CURR_USER "$PLUGIN_DEST_PATH"        
        if [ $? != 0 ]; then
            echo_with_flag "~/Library/Internet Plug-Ins mkdir failed"
        else
            echo_with_flag "~/Library/Internet Plug-Ins mkdir ok"
        fi
        cp -R -v "$PLUGIN_SOURCE_PATH" "$PLUGIN_DEST_PATH"
        if [ $? != 0 ]; then
            echo_with_flag "Plugin $PLUGIN_SOURCE_PATH copy failed"
        else
            echo_with_flag "Plugin $PLUGIN_SOURCE_PATH copy ok"
        fi
    
        echo_with_flag "set Safari preferences for plugin"
        PYTHON_COMMAND_SAFARI="setPlugInLoadPolicies()"
        python -c "$PYTHON_ENVIRONMENT$PYTHON_COMMAND_SAFARI"
    
        if [ $? != 0 ]; then
            echo_with_flag "set Safari preferences for plugin failed"
            return 1
        else
            # Safari, you're drunk, go home!
            defaults read com.apple.Safari ManagedPlugInLoadPolicies
            defaults read com.apple.Safari ManagedPlugInPolicies
        fi
        
        echo_with_flag "set Chrome preferences for plugin"
        PYTHON_COMMAND_CHROME="updateChromePreferencesFile()"
        python -c "$PYTHON_ENVIRONMENT$PYTHON_COMMAND_CHROME"
    
        if [ $? != 0 ]; then
            echo_with_flag "set Chrome preferences for plugin failed"
            return 1
        fi
    fi
}

plugin_uninstall()
{
    if [ "$PLUGIN_SAFARI_LOAD_POLICY_NAME" == "" ] ; then
        echo "no plugin"
    else
        rm $RMVERBARG -rf "$PLUGIN_DEST_PATH"
        if [ $? != 0 ]; then
            echo_with_flag "Plugin $PLUGIN_DEST_PATH remove failed"
        else
            echo_with_flag "Plugin $PLUGIN_DEST_PATH remove ok"
        fi    
    fi
}

check_and_install_or_plugin_uninstall()
{
	if [ "$PLUGIN_SAFARI_LOAD_POLICY_NAME" == "" ]; then
        echo "no plugin, do not call check_and_install_or_plugin_uninstall"
		return 1
	fi
	
    # if any extension is installed, install plugin
    chrome_extensionExists
    if [ $? == 0 ]; then
        plugin_install
        return 0
    fi
    
    safari_extensionExists
    if [ $? == 0 ]; then
        plugin_install
        return 0
    fi
    
    firefox_extensionExists
    if [ $? == 0 ]; then
        plugin_install
        return 0
    fi
    
    # no extensions installed, uninstall plugin
    plugin_uninstall
    
    return 1
}

### Script body ###
if [ "$#" -lt "2" ]; then
    echo_with_flag "Invalid arguments."
    exit 1
fi

_HOME="$_HOME"
if [ x"$4" != "x" ]
then
    _HOME="$4"
fi

### GLOBAL VARS ###
# find account name by user home dir ($USER and $SUDOUSER are not available)
# CURR_USER=`python -c "import os,pwd; print [x.pw_name for x in pwd.getpwall() if x.pw_dir == os.getenv('HOME')][0]"`
CURR_USER=$(basename $_HOME)
echo_with_flag "$CURR_USER"
CHROME_PREFERENCES_FILE_PATH="/Users/$CURR_USER/Library/Application Support/Google/Chrome/Default/Preferences"

PYTHON_ENVIRONMENT="""
import os.path
import plistlib
import sys
import xml.parsers.expat as expat
import commands
import json

def updateChromePreferencesFile():
    jsonFile = open('$CHROME_PREFERENCES_FILE_PATH', 'r')
    data = json.load(jsonFile)
    jsonFile.close()
    
    data['profile']['content_settings']['plugin_whitelist']['KPMPlugin'] = { 'plugin' : True }
    
    jsonFile = open('$CHROME_PREFERENCES_FILE_PATH', 'w+')
    jsonFile.write(json.dumps(data))
    jsonFile.close()

def setPlugInLoadPolicies():
    plistPath = '$SAFARI_PREFERENCES_PLIST_PATH'
    os.system('/usr/bin/plutil -convert xml1 %s' % plistPath )    
    try:
        preferencesPlist = plistlib.readPlist(plistPath)    
        if not(preferencesPlist.has_key('ManagedPlugInLoadPolicies')):
            preferencesPlist['ManagedPlugInLoadPolicies'] = {}
        if not(preferencesPlist.has_key('ManagedPlugInPolicies')):            
            preferencesPlist['ManagedPlugInPolicies'] = {}	    	
    except:
        preferencesPlist = {'ManagedPlugInLoadPolicies' : {},
                            'ManagedPlugInPolicies' : {}}

	preferencesPlist['ManagedPlugInLoadPolicies']['$PLUGIN_SAFARI_LOAD_POLICY_NAME'] = {
        'PlugInDisallowPromptBeforeUseDialog' : True,
        'PlugInFirstVisitPolicy' : 'PlugInPolicyAllowNoSecurityRestrictions'
    }
    preferencesPlist['ManagedPlugInPolicies']['$PLUGIN_SAFARI_LOAD_POLICY_NAME'] = {
        'PlugInDisallowPromptBeforeUseDialog' : True,
        'PlugInFirstVisitPolicy' : 'PlugInPolicyAllowNoSecurityRestrictions'
    }

    plistlib.writePlist(preferencesPlist, plistPath)
    return preferencesPlist
	

def readExtensionsPlistObject(path):
    os.system('/usr/bin/plutil -convert xml1 %s' % path )    
    try:
        extensionPlist = plistlib.readPlist(path)    
        if not(extensionPlist.has_key('Installed Extensions')):
        	extensionPlist['Installed Extensions'] = []                	
    except:
        extensionPlist = { 'Installed Extensions' : [],
                           'Version'              : 1 }
    return extensionPlist

def uninstallExtensionBody(extensionPlist, extensionArchiveName):
    flag = True
    while flag:
        flag = False    
        for anEntry in extensionPlist['Installed Extensions']:
            if anEntry['Archive File Name'] == extensionArchiveName:
                flag = True
                extensionPlist['Installed Extensions'].remove(anEntry)
    return extensionPlist

def uninstallExtension(extensionArchiveName):
    path = '$EXTZPLIST_PATH'
    extensionPlist = readExtensionsPlistObject(path)    	
    ###	
    extensionPlist = uninstallExtensionBody(extensionPlist, extensionArchiveName)
    ###
    plistlib.writePlist(extensionPlist, path)

def installExtension(extensionArchiveName, extensionBundleName):
    path = '$EXTZPLIST_PATH'
    extensionPlist = readExtensionsPlistObject(path)    	
    ###
    extensionPlist = uninstallExtensionBody(extensionPlist, extensionArchiveName)
    theEntry = {}
    theEntry['Archive File Name'] = extensionArchiveName
    theEntry['Bundle Directory Name'] = extensionBundleName
    theEntry['Enabled'] = True
    extensionPlist['Installed Extensions'].append(theEntry)
    ###
    plistlib.writePlist(extensionPlist, path)
"""
APP_ID=$3
SCRIPT_CMD=$2
SCRIPT_BROWSER=$1
if [ $1 == "safari" ]; then
    EXT_VERSION="$SAFARI_EXT_VERSION"
    EXT_SRC_PATH="$SAFARI_SOURCE_PATH_SHARED"
elif [ $1 == "chrome" ]; then
    EXT_VERSION="$CHROME_EXT_VERSION"
    EXT_SRC_PATH="$CHROME_SOURCE_PATH_SHARED"
elif [ $1 == "yandex" ]; then
    EXT_VERSION="$YANDEX_EXT_VERSION"
    EXT_SRC_PATH="$YANDEX_SOURCE_PATH_SHARED"
    SCRIPT_BROWSER="yandex"
elif [ $1 == "opera" ]; then
    EXT_VERSION="$OPERA_EXT_VERSION"
    EXT_SRC_PATH="$OPERA_SOURCE_PATH_SHARED"
    SCRIPT_BROWSER="opera"
elif [ $1 == "firefox" ]; then
    EXT_VERSION="$FIREFOX_EXT_VERSION"
    EXT_SRC_PATH="$FIREFOX_SOURCE_PATH_SHARED"
else
    EXT_VERSION="0.0"
    EXT_SRC_PATH=""
    SCRIPT_BROWSER="nobro"
fi

SCRIPT_DIR=$(dirname "$0")
OSVER=`sw_vers -productVersion | awk -F"." '{printf "%d.%d\n", $1, $2}'`
OSVERMINOR=`echo "$OSVER" | sed 's/\([^.]*\).\([^.]*\).*/\2/'`
SAFARI_VERSION_MAJOR=$(echo $(/usr/libexec/plistbuddy -c Print:CFBundleShortVersionString: "/Applications/Safari.app"/Contents/Info.plist) | sed 's/\([^.]*\).\([^.]*\).*/\1/')
SAFARI_VERSION_MINOR=$(echo $(/usr/libexec/plistbuddy -c Print:CFBundleShortVersionString: "/Applications/Safari.app"/Contents/Info.plist) | sed 's/\([^.]*\).\([^.]*\).*/\2/')

# Shared extensions location logic
# [TODO] actually check somewhere extension state in browser, not in our plist
if [ "$SCRIPT_CMD" == "install" ] || [ "$SCRIPT_CMD" == "uninstall" ] || [ "$SCRIPT_CMD" == "extensionExists" ]; then
    if [ "$5" == "SHARED" ] ; then
        "$SCRIPT_BROWSER"_"$SCRIPT_CMD"
    else
        # If browser exists, perform installation
        "$SCRIPT_BROWSER"_"exists"
        if [ "$?" == "0" ]; then
            python "$SCRIPT_DIR"/shared_extensions_installer.py "$SCRIPT_BROWSER" "$SCRIPT_CMD" "$EXT_VERSION" "$EXT_SRC_PATH" "$EXT_ID" "$APP_ID" "$SELF_NAME" "$_HOME"
            if [ "$?" == "0" ]; then
	   			_USER=$(basename "$_HOME")
	   			_GRP=$(id -g -n "$_USER")
	   			[ $? == 0 ] && chown -R "$_USER":"$_GRP" "$_HOME/Library/Application Support/Kaspersky Lab/Extensions/"
                exit 0
            else
                echo "shared_extensions_installer call failed"
	   			_USER=$(basename "$_HOME")
	   			_GRP=$(id -g -n "$_USER")
	   			[ $? == 0 ] && chown -R "$_USER":"$_GRP" "$_HOME/Library/Application Support/Kaspersky Lab/Extensions/"
                exit 1
            fi
        else
            echo "No $SCRIPT_BROWSER browser, do not call shared_extensions_isntaller.py"
            exit 1
        fi                
    fi
else
    "$SCRIPT_BROWSER"_"$SCRIPT_CMD"
fi

exit $?
